
# q-expansion of newform 4235.2.a.j, downloaded from the LMFDB on 27 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 4235
weight = 2
poly_data = [-1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1, -2], [-1, 1], [-1, 0], [-1, 0], [0, 0], [3, 1], [4, 1], [0, -2], [2, -4], [-4, 3], [0, 2], [-4, -2], [-4, 4], [-2, -2], [-6, 3], [-8, 4], [-6, -2], [-8, 2], [-8, 10], [5, 7], [8, -1], [-7, 11], [-5, 9], [16, -2], [-4, -7], [-4, -6], [-6, -7], [-4, -2], [-18, 4], [4, 4], [6, 2], [8, 0], [8, -12], [12, -4], [-13, -3], [-2, 1], [-8, -5], [-6, -10], [-12, 0], [13, -9], [18, 1], [12, -4], [15, 5], [4, 0], [-12, 0], [-18, 0], [19, -7], [8, -4], [7, -11], [-28, 2], [-16, 14], [-9, 21], [12, -18], [2, -6], [-4, 1], [-2, 10], [4, -14], [20, 2], [10, -12], [-2, -4], [-22, 9], [10, -12], [-6, -9], [-16, 18], [-18, 0], [-28, 4], [9, -17], [-12, -12], [-8, 4], [-22, 16], [1, -9], [21, -21], [9, -25], [-18, 18], [-6, 19], [9, 11], [-19, -9], [-15, 31], [-12, 17], [0, -2], [10, -28], [-7, -17], [6, -7], [-9, 13], [14, -6], [-26, 14], [15, -15], [14, -32], [26, -6], [12, -18], [-21, -7], [0, 14], [-18, 10], [28, -16], [-2, -13], [-5, 5], [-6, 20], [2, -10], [6, -3], [11, -3], [24, -26], [-8, -6], [-22, -1], [-20, -11], [-2, -21], [-4, 23], [31, -15], [-8, 11], [2, 15], [4, -12], [-10, 5], [-2, -24], [-6, 2], [-16, -4], [18, -17], [-13, 21], [-7, 11], [-42, -3], [-22, -10], [17, -13], [-26, 12], [-4, 16], [44, 3], [38, -2], [12, -24], [-6, -28], [31, -19], [-4, 16], [-27, -9], [-25, 37], [12, 0], [28, -14], [26, 7], [-12, 10], [10, 24], [2, 6], [-29, 1], [19, 13], [-19, 39], [5, -5], [12, 12], [24, -7], [-20, 0], [-12, -16], [8, 16], [40, -20], [33, -9], [14, -44], [-24, 8], [16, -8], [0, 34], [-2, -2], [6, -10], [-26, 3], [2, 16], [26, -17], [-22, 15], [8, -16], [41, -25], [18, -30], [-8, 26], [30, 8], [-20, 2], [-22, 12], [38, 2], [6, -21], [9, 3], [12, -23], [-4, 37], [53, -13], [34, 5], [-12, -12], [15, -15], [-34, -8], [30, -30], [-12, -12], [34, -1], [-18, 0], [27, -35], [8, -10], [32, -14], [-12, 30], [-22, -10], [36, -1], [-50, 10], [52, -10], [-38, -2], [12, -4], [5, 3], [24, -46], [8, 21], [-14, 46], [47, -7], [-48, 12], [-24, 36], [-24, 23], [-13, 33], [20, -42], [-3, 35], [-3, -21], [-14, 20], [50, 0], [-31, 43], [-42, -4], [-4, 8], [4, 17], [-16, 30], [10, -16], [41, -5], [-30, 54], [48, -14], [40, -18], [4, -30], [-31, 3], [-32, 12], [-46, 24], [-52, 22], [-12, 12], [36, -50], [24, -42], [-38, 40], [10, 8], [-31, 51], [-36, -16], [4, -28], [17, 31], [1, -33], [12, 0], [-32, 24], [55, -15], [-34, -14], [-24, -4], [5, -5], [-35, -1], [-30, 25], [-62, 21], [16, 0], [14, -32], [10, -25], [-18, -16], [-22, 32], [-12, 10], [34, -1], [-12, 37], [51, 9], [10, 26], [8, -16], [18, 14], [22, 3], [10, 24], [-24, 12], [4, -40], [38, -46], [-6, -40], [-34, 16], [-34, 11], [-39, 15], [58, 5], [-41, -3], [-14, 28], [-30, 2], [-60, 20], [-6, -36], [40, -17], [51, -27], [-48, 6], [-38, 52], [36, 8], [-48, 18], [-8, 18], [-32, 20], [22, -20], [33, -5], [-10, 57], [-43, -9], [33, -9], [-6, 14], [-16, -7], [12, -19], [-14, 38], [22, -11], [49, -13], [31, -39], [44, -12], [8, 16], [-24, 8], [48, -20], [-14, 14], [-24, 38], [-26, 46], [-52, 25], [-12, 40], [4, 32], [22, -46], [-14, -24], [10, -56], [72, 0], [-16, -21], [10, 2], [74, -19], [-26, -4], [-6, 8], [12, -44], [-54, 50], [35, 13], [-56, 38], [-48, 32], [-22, -26], [26, -5], [50, 24], [-23, 11], [0, 7], [41, -37], [80, -2], [38, -12], [-62, -10], [56, -30], [42, 7], [50, -12], [28, -63], [0, 50], [-6, -2], [42, -56], [4, 32], [26, 26], [24, -16], [-10, -3], [-35, 55], [14, -6], [-56, 58], [-35, -5], [6, 44], [32, 26], [26, -7], [-52, 8], [-44, 0], [-8, -11], [-33, -15], [54, -28], [-12, -8], [-24, -12], [-8, 22], [-46, 2], [6, 17], [-18, -38], [55, -27], [-42, 42], [-10, -2], [-6, -40], [46, -52], [-10, 4], [-44, 12], [4, 45], [-14, -15], [0, 25], [-32, 33], [16, -38], [28, 22], [-36, 6], [-40, 8], [-20, 4], [12, -56], [12, -58], [-44, 36], [4, 28], [66, -2], [36, -3], [-10, 20], [40, 1], [53, -73], [-4, -44], [8, -8], [-17, 57], [-38, 46], [-8, -12], [52, -24], [-19, -21], [-5, -11], [-10, 0], [-29, 45], [-40, 48], [-12, 6], [2, 22], [-62, 0], [56, 6], [26, 25], [15, -31], [15, -19], [-50, 20], [20, 24], [-7, 39], [-19, 15], [40, -24], [-54, 19], [54, -25], [-36, 30], [60, 12], [38, 6], [5, -17], [-32, 39], [68, 20], [-34, -14], [-18, 20], [-3, 31], [34, 8], [22, 4], [-38, 60], [-58, 56], [-42, 45], [-72, -4], [30, 4], [-2, 20], [-56, 12], [-18, 21], [32, -34], [-16, 44], [12, -48], [-30, 29], [-32, -17], [-42, 47], [28, -63], [-7, -1], [-34, -26], [6, -44], [23, 29], [22, -56], [14, -6], [48, -71], [68, -26], [42, 0]]
